#!/usr/bin/python
# -*- coding: utf-8 -*-

'''
Created on 2010-7-15

@author: GL
'''

from google.appengine.ext import webapp
from google.appengine.ext.webapp.util import run_wsgi_app
from google.appengine.ext.webapp import template
import base64
from urllib import urlencode
import copy
import logging
from gae_rproxy import rproxy
from gae_rproxy import settings
import os


def http_500(res, msg=''):
    res.set_status(500)
    res.out.write("""
        服务器的情绪不太稳定。 %s
    """ % msg)


class DefaultHandler(webapp.RequestHandler):

    def handle(self):
        try:
            self._handle()
        except rproxy.ProxyError, e:
            if e.err_code in ('cycle_proxy', 'empty_request'):
                path = os.path.join(os.path.dirname(__file__),
                                    '../templates/index.html')
                self.response.out.write(template.render(path, {
                    'center' : settings.CENTER, 
                    'title': settings.TITLE,
                    'logotext': settings.LOGOTEXT,}))
            elif e.err_code == 'fetch_error':
                http_500(self.response, 'error to download:%s'
                         % e.err_msg)
            else:
                logging.exception('error when handle')
                http_500(self.response)
        except:
            logging.exception('error when handle')
            http_500(self.response)

    def _handle(self):
        pass



class GetProxyUrlHandler(DefaultHandler):

    def _handle(self):
        gres = self.response
        q = base64.b64decode(self.request.get('url'))
        scheme = self.request.environ['wsgi.url_scheme']
        domain = self.request.environ['SERVER_NAME']
        proxied = rproxy.replacer.proxy_url(q, domain, scheme)
        if proxied != q:
            self.response.set_status(302)
            self.response.headers['location'] = proxied
            return
        headers = copy.copy(self.request.headers)
        headers['referer'] = 'http://www.google.com.hk'
        # http://www.google.com/search?btnI=I%27m+Feeling+Lucky&q=%s
        google_url = 'http://www.google.com.hk/search?%s' \
            % urlencode({'q': q})
        proxied_google_url = rproxy.replacer.proxy_url(google_url,
                domain, scheme)

        (   res,
            proxied_headers,
            proxied_data,
            _,
            _,
            _,
            ) = rproxy.get_proxied_response(proxied_google_url, 'GET',
                    headers)
        gres.set_status(res and res.status or 302)
        for (k, v) in proxied_headers.items():
            if 'set-cookie' == k.lower():
                gres.headers.add_header(k, v)
            else:
                gres.headers[k] = v
        gres.out.write(proxied_data)

    def get(self):
        return self.handle()

    def post(self):
        return self.handle()

class ProxyHandler(DefaultHandler):

    def _handle(self):
        greq = self.request
        gres = self.response

        (   res,
            proxied_headers,
            proxied_data,
            _,
            _,
            _,
            ) = rproxy.get_proxied_response(greq.url, greq.method,
                    greq.headers, greq.body)

        gres.set_status(res and res.status or 302)
        for (k, v) in proxied_headers.items():
            if 'set-cookie' == k.lower():
                gres.headers.add_header(k, v)
            else:
                gres.headers[k] = v
        gres.out.write(proxied_data)

    def get(self):
        return self.handle()

    def post(self):
        return self.handle()

    def put(self):
        return self.handle()

    def head(self):
        return self.handle()

    def options(self):
        return self.handle()

    def delete(self):
        return self.handle()

    def trace(self):
        return self.handle()


class BannerHandler(webapp.RequestHandler):

    def get(self):
        path = os.path.join(os.path.dirname(__file__),
                            '../templates/banner.html')
        self.response.out.write(template.render(path, {
            'center' : settings.CENTER}))


def main():
    logging.getLogger().setLevel(logging.DEBUG)
    application = webapp.WSGIApplication([
        (r'/_rpx_get_proxy_url', GetProxyUrlHandler), 
        (r'/_rpx_banner.html', BannerHandler),
        (r'/.*', ProxyHandler)
        ], debug=True)
    run_wsgi_app(application)


if __name__ == '__main__':
    main()
