# -*- coding: utf-8 -*-
'''
Created on 2010-7-18

@author: GL
'''
import niceurllib, settings
from wsgiref.headers import Headers
import copy
import logging
import re
import base64
try:
    from google.appengine.api.urlfetch_errors import DownloadError as FetchError
except ImportError:
    from httplib import HTTPException as FetchError



HOST_PATTERN = re.compile(r"^((?P<scheme>\w*)://)?(?P<host>[\w_\-\.]+[\w_\-]+\.[\w_\-]+)(?P<uri>/?.*$)")
#解析 rproxy url
RHOST_PATTERN = re.compile(r"^(?P<proxy_scheme>\w*)://(?P<server>[^/]*)/(?P<host>[^/]*)/(?P<scheme>http|https)(/(?P<uri>.*)|$)")
#替换 url 为 rproxy url

#Below is an URL from facebook html
#"src":"https:\/\/s-static.ak.facebook.com\/rsrc.php\/z3QPH\/p\/hash\/3orpr56t.js"
#action=\"/fIzzz\/ajax\/updatestatus.php\" is not work  must be 
#action=\"\/fIzzz\/ajax\/updatestatus.php\"
SRC_HOST_PATTERN = re.compile(r"(\s|\.|\,|'|\")(src|href|HREF|action|embed|url|server)(\\?['\"]?\s*[=:]\s*)(\\?'|\\?\"|&#39;|&apos;|&quot;)(http|https|)(:\\?/\\?/)([^\\/'\"]*)(\\?/?[^'\"]*?)(\\?'|\\?\"|&#39;|&apos;|&quot;)")
SRC_ROOT_PATTERN = re.compile(r"(\s|\.|\,|'|\")(src|href|action|embed|url)(\\?['\"]?\s*[=:]\s*)(\\?'|\\?\"|&#39;|&apos;|&quot;)(\\?/[^'\"]*)(\\?'|\\?\"|&#39;|&apos;|&quot;)")
#see this special one, url(//www.google.com/help/hc/images/picasa_cfheart.png)
CSS_HOST_PATTERN = re.compile(r"(\(\s*['\"]?\s*)(http|https|):?//([^/]*)(/[^\)]*?)(\s*['\"]?\s*\))")
CSS_ROOT_PATTERN = re.compile(r"([^\w]url\(\s*['\"]?\s*)(/[^\)]*)(\s*['\"]?\s*\))")
#background:url(/intl/en_com/images/srpr/logo1w.png)

class UnproxyInfo(object):
    def __init__(self):
        self.proxy_scheme = self.scheme = 'http'
        self.url = self.proxy_domain = self.host = self.path = \
                self.uri = self.alias = None
        self.debug = False

def offset_encode(txt, offset=3):
    if offset == 0:
        return txt
    txt2 = []
    for c in txt:
        i = ord(c)
        if i >= 97 and i <= 122:
            i += offset
            if i > 122:
                i -= 26
            if i < 97:
                i += 26
        txt2.append(chr(i))
    return ''.join(txt2)

class UrlReplacer(object):

    def _uri_split_replace(self, uri, fun, *args):
        chars = []
        i = 0
        run = True
        while i < len(uri) :
            if run:
                n = uri.find('/', i)
            if n < 0:
                n = uri.find('?', i)
                run = False
            if n < 0:
                n = len(uri)
            p = uri[i + 1:n]
            i = n
            chars.append(uri[i])
            chars.append(fun(p, *args))
        return ''.join(chars)

    def replace_query(self, uri, info):
        def repl(m):
            if m.group(3).startswith('http'):
                s = self.proxy_url(m.group(3), info.proxy_domain, info.proxy_scheme)
            elif re.match(r'\.(:\w{2,4})$', m.group(3)):
                s = ''
            else:
                return m.group(0)
            return ''.join([m.group(1), m.group(2), '=', s])
        return re.sub(r'(\?|&)([^&=]*)=([^&=]*)', repl, uri)


    def encode_uri(self, uri, offset=0):
        if offset == 0:
            return uri
        chars = []
        i = 0
        m = uri.find('?', i)
        if m < 0:m = len(uri)
        while i < len(uri) :
            n = uri.find('/', i)
            n = min(n, m)
            if n < i:
                n = len(uri)
            p = uri[i:n]
            i = n + 1

            if p.startswith('Z'):
                chars.append('XZ')
                chars.append(offset_encode(p[1:], offset))
            elif len(p) > 0:
                chars.append('Z')
                chars.append(offset_encode(p, offset))
            else:
                chars.append('')
            if n < len(uri):
                chars.append(uri[n])

        return ''.join(chars)

    def decode_uri(self, uri, offset=0):
        if offset == 0:
            return uri
        chars = []
        i = 0
        m = uri.find('?', i)
        if m < 0:m = len(uri)
        while i < len(uri) :
            n = uri.find('/', i)
            n = min(n, m)
            if n < i:
                n = len(uri)
            p = uri[i:n]
            i = n + 1
            if p.startswith('XZ'):
                chars.append('Z' + offset_encode(p[2:], offset))
            elif p.startswith('Z'):
                chars.append(offset_encode(p[1:], offset))
            else:
                chars.append(p)

            if n < len(uri):
                chars.append(uri[n])

        return ''.join(chars)

    def decode_host(self, alias):
        names = []
        i = 0
        l = len(alias)
        offset = 0
        _token_info = [0, 0]#_last, _found_symbol

        def token(pos):
            v = alias[_token_info[0]:pos]
            _token_info[0] = pos + 1
            _token_info[1] = 0
            return offset_encode(v, offset)

        while i < l:
            c = alias[i]
            if c == 'I':
                names.insert(0, self._get_host(token(i)))
                if i < l - 1:
                    names.insert(0, '.')
                offset = -3
            elif c == 'E':
                c = token(i)
                if len(c) > 0:
                    names.insert(0, c)
                offset = -3
            elif c == '.' or c == 'Z':
                c = token(i)
                if len(c) > 0:
                    names.insert(0, c)
                names.insert(0, '.')
            elif c == 'W':
                c = token(i)
                if len(c) > 0:
                    names.insert(0, c)
                names.insert(0, 'www')
            elif i == l - 1:
                c = token(l)
                names.insert(0, _token_info[1] and c or self._get_host(c))
            i += 1
        return ''.join(names)
#        m = re.match(r'^(.+)I(.+)$', alias)
#        if m:
#            return '%s.%s' % (m.group(1), self._get_host(m.group(2)))
#        return self._get_host(alias)

    def _encrypt_host(self, host, offset):
        return 'Z'.join([offset_encode(s, offset) for s in reversed(host.split('.'))])

    def encode_host(self, host):
        if host.find('.') < 0:
            return host
        m = re.match(r'^(.*)\.([^\.]+\.[^\.]+)$', host)
        if not m:
            g1 = None
            g2 = host
        else:
            g1 = m.group(1)
            g2 = m.group(2)
        alias = self._get_alias(g2)
        offset = 3
        if alias == g2:
            alias = self._encrypt_host(g2, offset)
            if not g1:
                return r'E%s' % alias
            else:
                return r'E%sZ%s' % (alias, self._encrypt_host(g1, offset))
        if g1:
            return r'%sI%s' % (alias, self._encrypt_host(g1, offset))
        return alias

    def _get_host(self, alias):
        return alias

    def _get_alias(self, host):
        return host

    def is_host_name(self, host):
        return re.match(r'^[\w-_]+(\.[\w-_]+)+$', host)

    def proxy_url(self, url, proxy_domain, proxy_scheme='http', info=None, debug=False):
        m = HOST_PATTERN.match(url)
        if m:
            uri = m.group('uri')
            return "%s://%s/%s%s/%s%s" % (proxy_scheme,
                                 proxy_domain,
                                 debug and 'D' or '',
                                 self.encode_host(m.group('host')),
                                 m.group('scheme') or 'http',
                                 len(uri) == 0 and '/' \
                                    or query_pattern.sub(get_repl(info, proxy_domain, proxy_scheme), uri))
        return url

    def unproxy_url(self, url):
        info = UnproxyInfo()
        m = RHOST_PATTERN.match(url)
        if not m:
            return None
        info.proxy_domain = m.group('server')
        info.proxy_scheme = m.group('proxy_scheme')
        info.path = info.alias = m.group('host')
        info.uri = unproxy_pattern.sub(get_unproxy_repl(info.proxy_domain), m.group('uri') or '')
        info.scheme = m.group('scheme')
        if info.alias.startswith('D'):
            #D - raw data ,no replacement for debug
            info.debug = True
            info.alias = info.alias[1:]
        info.host = self.decode_host(info.alias)
        info.url = "%s://%s/%s" % (info.scheme , info.host , info.uri)
        return info

    def proxy_cookie(self, cookie, host, proxy_scheme='http', proxy_domain=''):
        logging.debug(("old set-cookie:", cookie))
        #cookie patterns
        domain_pattern = re.compile(r'([;^]?\s*domain=)([^;]*)([;$]?)', re.I)
        path_pattern = re.compile(r"([;^]?\s*path=)([^;]*)([;$]?)", re.I)
        secure_pattern = re.compile(r'([;^]?\s*)(secure)([;$]?)', re.I)
        httponly_pattern = re.compile(r'([;^]?\s*)(httponly)([;$]?)', re.I)
        is_http = httponly_pattern.search(cookie)
        is_secure = secure_pattern.search(cookie)
        has_domain = domain_pattern.search(cookie)
        domain = has_domain and has_domain.group(2) or host
        if has_domain:
            cookie = domain_pattern.sub(r'\1%s\3' % proxy_domain, cookie)

        if settings.HTTPS != 'AUTO':
            if is_secure and proxy_scheme == 'http':
                cookie = secure_pattern.sub(r'\1HttpOnly\3', cookie)
            if is_http and proxy_scheme == 'https':
                cookie = httponly_pattern.sub(r'\1Secure\3', cookie)

        if not path_pattern.search(cookie):
            cookie = cookie + '; path=/'

        if domain.startswith('.'):
            path_repl = r'\1/\3'
        elif is_http:
            path_repl = r'\1/%s/http/\3' % self.encode_host(domain)
        elif is_secure:
            path_repl = r'\1/%s/https/\3' % self.encode_host(domain)
        else:
            path_repl = r'\1/%s/\3' % self.encode_host(domain)

        cookie = path_pattern.sub(path_repl, cookie)
        logging.debug(("new set-cookie:", cookie))
        return cookie

    def proxy_cookies(self, cookies, info):
        if isinstance(cookies, basestring):
            cookies = re.sub(r', ([^,;]+=)', r'\n\1', cookies)
            cookies = cookies.splitlines()
        #将cookie域名置空
        return [self.proxy_cookie(cookie, info.host, info.proxy_scheme, info.proxy_domain) for cookie in cookies]

    def replace_data(self, data, patterns, info, ref_info=None):
        for t in patterns:
            if re.match(t[0], info.url):
                data = t[1](self, data, info, ref_info)
        return data

    def proxy_data(self, data, content_type, info, ref_info=None):
        #content_type, proxy_domain, host, path, ref_path=None, url=''
        if not content_type:
            return data

        if content_type.find("javascript") >= 0:
            root_path = ref_info and ref_info.path or info.path
            root_scheme = ref_info and ref_info.scheme or info.scheme
        elif content_type.find("text") >= 0:
            root_path = info.path
            root_scheme = info.scheme
        else:
            return data

        SRC_HOST_REPL = lambda m :''.join((m.group(1), m.group(2), m.group(3), m.group(4),
                                           info.proxy_scheme, m.group(6), info.proxy_domain,
                                           m.group(6) == '://' and '/' or r'\/',
                                           self.encode_host(m.group(7)),
                                           m.group(6) == '://' and '/' or r'\/',
                                           m.group(5), self.encode_uri(m.group(8)),
                                           m.group(9),
                                           ))

        SRC_ROOT_REPL = lambda m : ''.join((m.group(1), m.group(2), m.group(3), m.group(4),
                                            m.group(5).startswith(r'\/') and r'\/' or '/',
                                            root_path.replace('/', r'\/'),
                                            m.group(5).startswith(r'\/') and r'\/' or '/',
                                            root_scheme,
                                            self.encode_uri(m.group(5)),
                                            m.group(6),
                                            ))

        CSS_HOST_REPL = lambda m : ''.join((m.group(1), info.proxy_scheme, '://', info.proxy_domain, '/',
                                            self.encode_host(m.group(3)), '/',
                                            len(m.group(2)) > 0 and m.group(2) or info.scheme,
                                            self.encode_uri(m.group(4)),
                                            m.group(5),))
        CSS_ROOT_REPL = lambda m : ''.join((m.group(1) , '/', root_path, '/',
                                            root_scheme, self.encode_uri(m.group(2)) , m.group(3),))

        data = SRC_HOST_PATTERN.sub(SRC_HOST_REPL, data)
        data = SRC_ROOT_PATTERN.sub(SRC_ROOT_REPL, data)
        data = CSS_HOST_PATTERN.sub(CSS_HOST_REPL, data)
        data = CSS_ROOT_PATTERN.sub(CSS_ROOT_REPL, data)
        data = self.replace_data(data, settings.EX_REPLACE, info, ref_info)
        return data

class CustomUrlReplacer(UrlReplacer):
    def __init__(self):
        self._host2alias = {}
        self._alias2host = {}

    def add_hostalias(self, host, alias):
        self._alias2host[alias] = host
        self._host2alias[host] = alias

    def _get_host(self, alias):
        host = self._alias2host.get(alias)
        if not host:
            host = self._get_host_by_alias(alias) or alias
            self.add_hostalias(host, alias)
        return host

    def _get_alias(self, host):
        alias = self._host2alias.get(host)
        if not alias:
            alias = self._get_host_by_alias(host) or host
            self.add_hostalias(host, alias)
        return alias
    def _get_host_alias(self, host):
        return None

    def _get_host_by_alias(self, alias):
        return None

class DefaultUrlReplacer(CustomUrlReplacer):
    def __init__(self):
        CustomUrlReplacer.__init__(self)
        for k, v in settings.ALIAS.items():
            self.add_hostalias(v, k)

class ProxyError(Exception):
    def __init__(self, err_code, err_msg=None):
        self.err_code = err_code
        self.err_msg = err_msg
    def __str__(self):
        return repr(self.err_code, self.err_msg)



def _build_pattern(debug=True):
    from gae_rproxy.genre import any_of, cap, le1, capjoin, symbol, maybe, gen, join, if_group, may, genre
    query_anchor = r"[\?#][\w\.\-_\\:;/%&=\?#]*"
    path = r"(?P<sep>\\/|/)[\w\.:\-_\\/%&=]*"
    domain = r"[\w_\-\.]+"
    port = r":(?P<port>\d+)"
    domain_port = join(cap(domain, 'domain'), le1(port))
    path_query = join(cap(path, 'path'), le1(query_anchor, 'query'))
    domain_path = join(maybe(domain_port), maybe(path_query))
    http_domain = r"(?P<scheme>http|https)(?P<sep_1>:\\?/\\?/)" + domain_path
    slash_domain = r"(?P<sep_1>//)" + domain_path
    quots = r"(\\?'|\\?\"|&#39;|&apos;|&quot;)"
    attrs = r"(src|href|HREF|action|embed|url|server|uri)"

    operators = r"\s*(:|=|==|!=)\s*"
    funcs = r"(url|location\.replace|AsyncSignal|post|request|set%s)" % attrs

    #css 可以无引号形式,以//开头形式    :url(//www.google.com/....)
    patterns = [gen(
                    {"prefix":(r"[^\w]", r"url", r"\s*\(\s*['\"]?",)},
                    r"(?P<sep_1>//)" , domain_port, path_query,
                    {"suffix":'.?'},
#                     {"suffix": r"\s*['\"]?\s*\)"}
                )]
    #括号、引号所包含的 http://....
    patterns.append(gen(
                        {"prefix":r"\\?'|\\?\"|&#39;|&apos;|&quot;|\(\s*"},
                        r"(?P<scheme>http|https)(?P<sep_1>:\\?/\\?/)", domain_port,
                        [path_query],
                        {"suffix":r"\\?'|\\?\"|&#39;|&apos;|&quot;|\s*\)"},
                    ))
    #无http://domain，以 / 或 \/ 开头的形式
    patterns.append(gen(
         {"prefix":(symbol, [
                        ([quots], attrs, [quots], operators, quots,), #属性，有操作符
                     (funcs, r"\s*\(([^\)]*\,|\s*)['\"]",), #函数，有引号
                     (r"url", r"\s*\(\s*",), #css，无引号
                     ])},
         path_query,
         {"suffix":r"\\?'|\\?\"|&#39;|&apos;|&quot;|\s*\)"},
     ))

    patterns = [re.compile(p, re.I) for p in patterns]

#http%3A%2F%2Fwww.google.com%2Fabce.jpg
#http%3A%2F%2Fwww.google.com%2Fabce.jpg%3Fname%3Dvalue%26name2%3Dvalue2
#    qname = r'(?P<prefix>(continue|href|referer|action|from|back)=)'
#    {'domain':r'[\w\-\_]+\.(com|net|org|biz|info|mobi|edu|gov|asia|name|tel|([a-z][a-z]))'},
#    {'suffix':r'(%[\dA-F][\dA-F][^&]*|(&|$))'},
    query_pattern = genre({'prefix':'(^|%[\dA-F][\dA-F]|=|\,|\|)'},
                            {'scheme':r'http|https'},
                            {'sep_1':[r'%3A%2F%2F','://']},
                            {'domain':r'([\w\.\-\_]+)'},
                            {'path':r'((?P<sep>%2F|/)[\w\-_\./]*)'},
                            {'suffix':r'($|&|%|=|\,|\|)'},
                        )
    unproxy_pattern = genre({'prefix':'(^|[^\w])'},
                            {'proxy_scheme':r'http|https'},
                            {'sep_1':[r'%3A%2F%2F', '://', ':\/\/']},
                            {'proxy_domain':r'([\w\.:\-\_]+)'},
                            r'(?P<sep>%2F|/|\/)(?P<domain>[\w\-_\.]+)',
                            r'(%2F|/|\/)(?P<scheme>http|https)',
                            r'(?P<path>(%2F|/|\/)[\w\.\-_\\:/%]*)',
                            r'(?P<query>[\?#].*)?',#r'(?P<query>[\?#][\w\.\-_\\:/%&=\?#]*)?',
                            r'(?P<suffix>.|[^\w\.\-_\\:/%&=\?#]|$)',
                             )
    #page=%2FfIzzz%2Fhttp%2FTrueBlood
    unproxy_post_patterns = [genre(
                                 {'prefix':'[^\w]'},
                            {'proxy_scheme':r'http|https'},
                            {'sep_1':r'%3A%2F%2F'},
                            {'proxy_domain':r'([\w\.:\-\_]+)'},
                            r'(?P<sep>%2F|/|\/)(?P<domain>[\w\-_\.]+)',
                            r'(%2F|/|\/)(?P<scheme>http|https)',
                            r'(?P<path>(%2F|/|\/)[\w\.\-_\\:/%]*)',
#                            r'(?P<query>[\?#][\w\.\-_\\:/%]*)?',
                            r'(?P<suffix>&|=|\||\,|\?|#|$)',
                                 ),
                        genre(
                                 {'prefix':'(page|continue|href|location|referer|from|action)='},
#                            {'proxy_scheme':r'http|https'},
#                            {'sep_1':[r'%3A%2F%2F', '://', ':\/\/']},
#                            {'proxy_domain':r'([\w\.:\-\_]+)'},
                            r'(?P<sep>%2F|/|\/)(?P<domain>[\w\-_\.]+)',
                            r'(%2F|/|\/)(?P<scheme>http|https)',
                            r'(?P<path>(%2F|/|\/)[\w\.\-_\\:/%]*)',
#                            r'(?P<query>[\?#][\w\.\-_\\:/%&=\?#]*)?',
                            r'(?P<suffix>[^\w\.\-_\\:/%&=\?#]|$)',
                                 )]


    return patterns, query_pattern, unproxy_pattern, unproxy_post_patterns

patterns, query_pattern, unproxy_pattern, unproxy_post_patterns = _build_pattern()
del _build_pattern
replacer = DefaultUrlReplacer()

def get_unproxy_repl(proxy_domain=None):
    def repl(m):
        if proxy_domain and proxy_domain != m.group('proxy_domain'):
            return m.group(0)
        d = m.groupdict()
        query = d.get('query')
        return ''.join([d.get('prefix'),
                d.get('scheme'),
                d.get('sep_1'),
                replacer.decode_host(m.group('domain')),
                d.get('path'),
                query and unproxy_pattern.sub(repl, query) or '',
                d.get('suffix', '')
                ])
    return repl

def get_unproxy_post_repl(encoded_domain=None):
    def unproxy_post_repl(m):
        d = m.groupdict()
#        if encoded_domain and m.group('domain') != encoded_domain:
#            return m.group(0)
        return ''.join([
                d.get('prefix'),
                d.get('path'),
                d.get('suffix', '')
                ])
    return unproxy_post_repl

def get_repl(info=None, proxy_domain=None, proxy_scheme='http'):
    if info == None:
        info = UnproxyInfo()
        info.proxy_domain = proxy_domain
        info.proxy_scheme = proxy_scheme

    def repl(m):
        d = m.groupdict()
        domain = d.get('domain')
        path = d.get('path')
        query = d.get('query')
        if not (domain or path):
            return m.group(0)
        path = path or '/'
        if path.startswith(r'//%s/' % info.proxy_domain) or \
            path.startswith(r'\/\/%s\/' % info.proxy_domain):
            return m.group(0)

        if domain and domain == info.proxy_domain:
            return m.group(0)

        if path.find('ajax') > 0:
            return m.group(0)
        root_path = (domain == None or domain == info.host) and info.path or domain and replacer.encode_host(domain)

        scheme = d.get('scheme')
        sep1 = d.get('sep_1')
        sep = d.get('sep') or '/'
        return ''.join([d.get('prefix'),
                scheme and info.proxy_scheme or '',
                sep1 or '',
                domain and info.proxy_domain or '',
                sep,
                root_path,
                sep,
                scheme or info.scheme,
                path,
                query and query_pattern.sub(repl, query) or '',
                d.get('suffix') or ''
                ])

    return repl

def unproxy_post_data(data, proxy_domain=None, encoded_domain=None):
    data, n1 = unproxy_post_patterns[0].subn(get_unproxy_repl(proxy_domain), data)
    data, n2 = unproxy_post_patterns[1].subn(get_unproxy_post_repl(encoded_domain), data)
    return data, n1 or n2


def proxy_meta(data, info, is_ajax, content_type):
    if is_ajax or content_type and content_type.find('html') <= 0:
        return data

    repl = base64.b64encode(info.url)
    repl = r'\g<0><meta name="rproxyurl" content="%s"/>' % repl
    pattern = re.compile(r'<head\s*>', re.I | re.M)
    data = pattern.sub(repl, data)

    repl = r'<script>var _rpx_center="' + settings.CENTER + \
        r'"</script><script src="http://' + settings.CENTER + \
        r'/_rpx_static/rproxyclient.js"></script>'
    pattern = re.compile(r'</body>', re.I | re.M)
    data = pattern.sub(repl + '\g<0>', data)

    if not pattern.search(data):
        data = data + repl

    return data



def proxy_data_2(data, info, ref_info, content_type='text/html'):
    if not content_type:
        return data
    if content_type.find("javascript") >= 0:
        info = ref_info or info
    elif content_type.find("text") < 0:
        return data

    encode_query = lambda q : query_pattern.sub(repl, q)

    def repl(m):
        r = m.group(0)
        d = m.groupdict()
        domain = d.get('domain')
        path = d.get('path')
        query = d.get('query')
        if not (domain or path):
            return m.group(0)
        path = path or '/'
        if path.startswith(r'//%s/' % info.proxy_domain) or \
            path.startswith(r'\/\/%s\/' % info.proxy_domain):
            return m.group(0)
        if domain:
            if domain == info.proxy_domain:
                return m.group(0)

        root_path = domain and domain != info.host and replacer.encode_host(domain) or info.path

        scheme = d.get('scheme')
        sep1 = d.get('sep_1')
        sep = d.get('sep', '/')
        return ''.join([d.get('prefix'),
                scheme and info.proxy_scheme or '',
                sep1 or '',
                domain and info.proxy_domain or '',
                sep,
                root_path,
                sep,
                scheme or info.scheme,
                path,
                query and encode_query(query) or '',
                d.get('suffix', '')
                ])
    global patterns
    for p in patterns:
        #要避免循环替换，同样一个标签，两个pattern都符合，则会导致循环替换
        data = p.sub(get_repl(info), data)
    data = replacer.replace_data(data, settings.EX_REPLACE, info, ref_info)
    match = re.search(r'charset\s*=\s*(\S+)', content_type)
    encoding = match and match.group(1)
    try:
        import anticrab
        return anticrab.anti_crab(data, content_type, encoding)
    except:
        return data

script_pattern = re.compile(r'(?P<prefix>.*<script[^>]*>)(?P<script>(?!</script>).*)(?P<suffix></script>.*)', re.M)
unicode_pattern = re.compile(r'[^x00-xff]+')
HAS_UTF8 = re.compile(r'[\x80-\xff]')

def get_proxied_response(proxied_url, method, headers={}, body=None):
    opener = niceurllib.SimpleURLopener()
    logging.debug("getting proxied response of: %s", proxied_url)
    proxied_headers = Headers([])
    info = replacer.unproxy_url(proxied_url)
    #TODO: judge is ajax
    is_ajax = False

    if not (info and re.match(r'\w+\..*', info.host) and info.scheme in ('http', 'https')):
        m = re.match(r"^(?P<scheme>\w*)://(?P<server>[^/]*)/(?P<url>.*)" , proxied_url)
        assert m
        url = m.group('url')

        ref_url = headers.get('referer')
        flag = False
        if ref_url:
            #is ajax   # X-Requested-With:    XMLHttpRequest
            ref_info = replacer.unproxy_url(ref_url)
            if ref_info:
                if ref_info.host == ref_info.proxy_domain:
                    raise ProxyError('cycle_proxy')
                if ref_info.proxy_domain == m.group('server'):
                    info = copy.copy(ref_info)
                    info.url = '%s://%s/%s' % (info.scheme, info.host, url)
                    flag = True

        if not flag:
            if len(url) == 0:
                raise ProxyError('empty_request')
            if url.startswith("%24"):
                url = 'https://' + url[3:]
            else:
                url = 'http://' + url
            url = replacer.proxy_url(url, m.group('server'), m.group('scheme'))
            proxied_headers.add_header('location', url)
            return None, proxied_headers, r'''
    <A HREF="%s">You have been redirected</A>
                ''' % url, None, None, None


    ref_info = None
    for k, v in headers.items():
        logging.debug((k, v))
        if "host" == k.lower():
            v = info.host
        elif "referer" == k.lower():
            ref_info = replacer.unproxy_url(v)
            if ref_info:
                v = ref_info.url
                proxied_headers.add_header('x-rpx-referer', v)
        opener.addheader(k, v)
    logging.debug("fectching:%s", info.url)
    logging.debug(dir(info.url))
    proxied_headers.add_header('x-rpx-url', info.url)
    try:
        if body and len(body) > 0:
            logging.info("fetch by post method")
            body, n = unproxy_post_data(body, info.proxy_domain, info.alias)
            if n:
                logging.info('request.body.unproxyed:\n%s', body)
            res = opener.open(info.url, body)
        else:
            res = opener.open(info.url)
    except FetchError:
        logging.debug("error fetch url:%s", info.url)
        logging.debug("request body:%s", body, exc_info=1)
        raise ProxyError('fetch_error', info.url)

    logging.debug("response.STATUS:%d", res.status)
    logging.debug("response.headers:")
    #防止GAE强制加上 no-cache 的header
    proxied_headers['cache-control'] = 'private'
    for k, v in res.headers.items():
        logging.debug((k,v))
        k1 = k.lower()
        if k1 == 'set-cookie':
            for cookie in replacer.proxy_cookies(v, info):
                proxied_headers.add_header(k, cookie)
        elif k1 == 'location':
            proxied_headers.add_header('x-rpx-location', v)
            v = replacer.proxy_url(v, info.proxy_domain, info.proxy_scheme, info)
            proxied_headers[k] = v
        elif k1 == 'status':
            logging.debug("ignore:%s:%s", k, v)
            #这个status可能到导致了302跳转失败
            #twitter的cache-control有点问题
            continue
        else:
            #headers.addheader not work for content-type
            proxied_headers[k] = v

    content_type = res.headers.get("content-type")
    logging.debug("content-type:%s", content_type)
    data = res.read()
    proxied_data = info.debug and data or proxy_data_2(data, info, ref_info, content_type)
    proxied_data = proxy_meta(proxied_data, info, is_ajax, content_type)
    return res, proxied_headers, proxied_data, data, info, ref_info
