# -*- coding: utf-8 -*-
'''
Created on 2010-7-19

@author: GL
'''
from gae_rproxy import rproxy, settings
from gae_rproxy.rproxy import UnproxyInfo, proxy_data_2, unproxy_post_data
import re
import unittest

class URLReplacerTestCase(unittest.TestCase):

    def setUp(self):
        self.replacer = rproxy.UrlReplacer()

    def test_uri_encode(self):
        uri = '/google/test/uri/?name=value&xx=http://google.com'
        encoded = self.replacer.encode_uri(uri)
        assert uri == self.replacer.decode_uri(encoded)
        uri = 'google/ZXZoo/XZ?name=value&xx=http://google.com'
        encoded = self.replacer.encode_uri(uri)
        assert uri == self.replacer.decode_uri(encoded)
        uri = 'google/test/Zoo?'
        encoded = self.replacer.encode_uri(uri)
        assert uri == self.replacer.decode_uri(encoded)
        uri = 'Zoo/xxx'
        encoded = self.replacer.encode_uri(uri)
        assert uri == self.replacer.decode_uri(encoded)
        uri = 'zoo'
        encoded = self.replacer.encode_uri(uri)
        assert uri == self.replacer.decode_uri(encoded)

    def test_proxy_url(self):
        url = "http://www.google.com/images/images.png"
        encoded = self.replacer.proxy_url(url, "localhost:9999")
        assert url == self.replacer.unproxy_url(encoded).url
        info = self.replacer.unproxy_url(encoded)
        url = "http://www.google.com/images/images.png?continue=http%3A%2F%2Fwww.google.com%2Fabce.jpg%3Fname%3Dvalue%26name2%3Dvalue2&follow=http%3A%2F%2Fwww.google.com%2Fabce.jpg%3Fname%3Dvalue%26name2%3Dvalue2"
        encoded = self.replacer.proxy_url(url, "localhost:9999", "http", info, True)
        decoded = self.replacer.unproxy_url(encoded).url
        print encoded
        print  url
        print decoded
        assert encoded.find('google.com')<0
        assert url == decoded
        url = "http://www.google.com/images/images.png?continue=http://groups.google.com/group/python-tornado/msg/a&followup=http://groups.google.com/group/python-tornado/msg/b"
        encoded = self.replacer.proxy_url(url, "localhost:9999", "http", info, True)
        decoded = self.replacer.unproxy_url(encoded).url
        print encoded
        print  url
        print decoded
        assert encoded.find('groups.google.com')<0
        assert url == decoded
        
    def test_proxy_data(self):
        info = UnproxyInfo()
        info.url = r'http://www.facebook.com/blabla'
        info.proxy_domain = 'proxyserver.com'
        info.path = r'fIzzz'
        settings.HTTPS = 'NEVER'
        info.proxy_protocal = "http"
        ref_info = UnproxyInfo()
        ref_info.path = "twitter.com"
        raw_data = r'''
double slash css:
 url (//www.google.com/help/hc/images/picasa_cfheart.png?continue=%23%2Dwww.facebooke.com%2A%2e)
 url( "//www.google.com/help/hc/images/picasa_cfheart.png" )

with protocal:
"src":"https:\/\/s-static.ak.facebook.com\/rsrc.php\/z3QPH\/p\/hash\/3orpr56t.js?name=value"
"src"= "https:\/\/s-static.ak.facebook.com\/rsrc.php\/z3QPH\/p\/hash\/3orpr56t.js?name=value"
"src":"https:\/\/s-static.ak.facebook.com\/rsrc.php\/z3QPH\/p\/hash\/3orpr56t.js"
href=\"http:\/\/www.facebook.com\/profile.php?id=520433523&amp;ref=pymk\"
 replace("https://facebook.com/mypath/my.img")
href="http://www.facebook.com/xxxx"
url(https://facebook.com/mypath/my.img)
url(http://www.google.com/mypath/my.img?name=value)

without protocal:
--1 attribute:
action="/mypath/myurl?asdfasdf&asdfasdf=nnn"
href=\"\/ajax\/updatestatus.php?name=value\" is not work  must be
src=\"\/ajax\/updatestatus.php\" is not work  must be

--2 functions:
 url (  '/mypath/my.img'  )
 url (  '/ajax/something.php'  ) 
 location.replace ("/mypath/my.img") 
 .setURI('\/ajax\/feed_tracking_pagecache.php')
 AsyncSignal(something,'\/ajax\/feed\/save_view_state.php?name=value',data)
 
--3 css without quot without protocal: 
 url( /mypath/my.jpg ) 

--4 query_param
href="http://www.google.com/login?continue=http%3A%2F%2Fwww.google.com%2Fabce.jpg%3Fname%3Dvalue%26name2%3Dvalue2"

below should not be parsed. 
 http://www.google.com
 http://www.google.com/asdfasdf
 http://www.google.com/asdfas/?afasdf
 /asfasdf/asdfa
 /asdfad/asfas
        '''
        data = proxy_data_2(raw_data, info, ref_info)
        print data

    def test_unproxy_post_data(self):
        data = "page=%2FfIzzz%2Fhttp%2FTrueBlood&queryId=0&tab=wall&locations[0]=sidebar_ads&photo_refresh=no&cache=1&post_form_id=a2e5028c868921d3563664565a842b0b&fb_dtsg=62bX8&post_form_id_source=AsyncRequest"
        unproxied = unproxy_post_data(data, proxy_domain="localhost:9999", encoded_domain="fIzzz")
        print  unproxied
        data = "page=%2FfIzzz%2Fhttp%2Fprofile.php&queryId=100000235978472&tab=info&locations[0]=sidebar_ads&photo_refresh=no&cache=1&post_form_id=a2e5028c868921d3563664565a842b0b&fb_dtsg=62bX8&post_form_id_source=AsyncRequest"
        unproxied = unproxy_post_data(data, proxy_domain="localhost:9999", encoded_domain="fIzzz")
        print  unproxied
        data = "ltmpl=homepage&continue=http%3A%2F%2Ffuck-gfw4ever.appspot.com%2FgIgrfv%2Fhttp%2F&followup=http%3A%2F%2Ffuck-gfw4ever.appspot.com%2FgIgrfv%2Fhttp%2F&service=writely&dsh=4872966636056716396&ltmpl=homepage&ltmpl=homepage&timeStmp=&secTok=&GALX=eGkTW-VAjn0&Email=guileen&Passwd=a7878748&PersistentCookie=yes&rmShown=1&signIn=Sign+in&asts="
        unproxied = unproxy_post_data(data, proxy_domain="fuck-gfw4ever.appspot.com", encoded_domain="gIzzz")
        print  unproxied
        
    def test_proxy_cookie(self):
        return
        info = UnproxyInfo()
        info.proxy_domain = 'proxyserver.com'
        info.host = 'fuck-gfw.com'


        cookie = self.replacer.proxy_cookie("name = value; domain = fuck - gfw.com; path = / ; httponly", info)
        alias = self.replacer.get_alias('fuck-gfw.com').replace('.', r'\.')
        assert re.match(r'^name=value;(.*;\s*)?path=/%s/(;.*)?' % alias , cookie)

        cookie = self.replacer.proxy_cookie("name = value; domain = fuck - gfw.com; path = / path / ; httponly", info)
        assert re.match(r'^name=value;(.*;\s*)?path=/%s/http/path/(;.*)?' % alias, cookie)

        cookie = self.replacer.proxy_cookie("name = value; domain = fuck - gfw.com; path = / path / ; secure", info)
        assert re.match(r'^name=value;(.*;\s*)?path=/%s(;.*)?' % alias, cookie)

        cookie = self.replacer.proxy_cookie("name = value; domain = .fuck - gfw.com; path = / path / ; ", info)
        assert re.match(r'^name=value;(.*;\s*)?path=/(;.*)?' , cookie)

    def test_encode_host(self):
        alias0 = self.replacer.encode_host("twitter.com")
        aliasno = self.replacer.encode_host("noalias.cn")
        alias1 = self.replacer.encode_host("www.twitter.com")
        alias2 = self.replacer.encode_host("sub2.twitter.com")
        alias11 = self.replacer.encode_host("sub1.www.twitter.com")
        alias12 = self.replacer.encode_host("sub2.www.twitter.com")
        assert alias1.startswith(alias0) and len(alias1) > len(alias0)
        assert alias2.startswith(alias0) and len(alias2) > len(alias0)
        assert alias11.startswith(alias1) and len(alias11) > len(alias1)
        assert alias12.startswith(alias1) and len(alias12) > len(alias1)
        assert alias1 != alias2
        host0 = self.replacer.decode_host(alias0)
        assert host0 == "twitter.com"
        assert self.replacer.decode_host(alias1) == "www.twitter.com"
        assert self.replacer.decode_host(alias2) == "sub2.twitter.com"
        assert self.replacer.decode_host(alias11) == "sub1.www.twitter.com"
        assert self.replacer.decode_host(alias12) == "sub2.www.twitter.com"

    def test_decode_host(self):
        print self.replacer.decode_host('www.facebook.com')

    def test_anti_crab(self):
        html=u"""
        中文
        <script type="text/javascript">
        a="中文"
        </script>
        中文
        <form>
        中文 english
        <script>
        中文
        English
        </script>
        </form>
        """
        print rproxy.anti_crab(html)
"""
(?P < prefix > [ ^ \w](\\?'|\\?\"|&#39;|&apos;|&quot;)?
(src|href|HREF|action|embed|url|server)
(\\?' | \\?\"|&#39;|&apos;|&quot;)?\s*(:|=|==|!=)\s*(\\?'|\\?\"|&#39;|&apos;|&quot;)
|[^\w](url|replace)\s*\(\s*['\"]?)
((?P<protocal>http|https)(?P<sep_1>:\\?/\\?/)
(?P<domain>[\w_\-\.]+)
(:(?P<port>\d+))?)?
(?P<path>(?(domain)((?:\\/|/)[\w\-_\\/%&=]*)?|(?:\\/|/)[\w\-_\\/%&=]*))
(?P<query>[\?#][\w\-_\\:/%&=\?#]*)?
(?P<suffix>\))

"""
    